@@ -7,6 +7,7 @@ import tarfile
 import zipfile
 from pathlib import Path
 from platform import system, architecture
+from typing import List
 from urllib.parse import urlparse
 from urllib.request import urlretrieve
 from blake3 import blake3
@@ -19,16 +20,12 @@ from concurrent.futures import ThreadPoolExecutor
 
 __all__ = [
     "install",
-    "fpcalc_bin",
-    "ffprobe_bin",
-    "ffmpeg_bin",
-    "fpcalc_install",
-    "exiv2_bin",
-    "exiv2json_bin",
-    "ipfs_bin",
-    "ipfs_install",
-    "java_bin",
-    "tika_bin",
+    "run_ffmpeg",
+    "run_fpcalc",
+    "run_exiv2",
+    "run_exiv2json",
+    "run_ipfs",
+    "run_tika",
 ]
 
 BASE_VERSION = "1.0.0"
@@ -46,19 +43,6 @@ IPFS_CHECKSUMS = {
     "darwin-64": "3797fd0e6d5f922c095a12d860baccb49d90cef74accf49d219d4cea1b0d2bd7",
 }
 
-FFPROBE_VERSION = "4.4.1"
-FFPROBE_URLS = {
-    "windows-64": f"{BASE_URL}/ffprobe-{FFPROBE_VERSION}-win-64.zip",
-    "linux-64": f"{BASE_URL}/ffprobe-{FFPROBE_VERSION}-linux-64.zip",
-    "darwin-64": f"{BASE_URL}/ffprobe-{FFPROBE_VERSION}-osx-64.zip",
-}
-FFPROBE_CHECKSUMS = {
-    "windows-64": "6c6c7d49465f70f3a4c60dc2d5aeddb4049527c82ee6e2b4c6ad0a2a9fc9188e",
-    "linux-64": "bfa86d00341cacbbcaad6c38c706ad1df9f268b1d1e5a1f19206ab47a95aad8f",
-    "darwin-64": "1904fdf6d0250e3c44aa5f44b0bf31033b40329f1c3e51d3d06438273174506d",
-}
-
-
 FFMPEG_VERSION = "4.4.1"
 FFMPEG_URLS = {
     "windows-64": f"{BASE_URL}/ffmpeg-{FFMPEG_VERSION}-win-64.zip",
@@ -87,27 +71,27 @@ TIKA_VERSION = "2.3.0"
 TIKA_URL = f"{BASE_URL}/tika-app-{TIKA_VERSION}.jar"
 TIKA_CHECKSUM = "e3f6ff0841b9014333fc6de4b849704384abf362100edfa573a6e4104b654491"
 
-EXIV2_VERSION = "0.27.5"
+EXIV2_VERSION = "0.27.2"
 EXIV2_URLS = {
-    "windows-64": f"{BASE_URL}/exiv2-{EXIV2_VERSION}-2019msvc64.zip",
+    "windows-64": f"{BASE_URL}/exiv2-{EXIV2_VERSION}-2017msvc64.zip",
     "linux-64": f"{BASE_URL}/exiv2-{EXIV2_VERSION}-Linux64.tar.gz",
     "darwin-64": f"{BASE_URL}/exiv2-{EXIV2_VERSION}-Darwin.tar.gz",
 }
 
 EXIV2_CHECKSUMS = {
-    "windows-64": "3e00112648ed98a60a381fc3c6dd10ec263b1d56dec4f07dce86a7736517ebcd",
-    "linux-64": "6c6339f7f575ed794c4669e7eab4ef400a6cf7981f78ea26fc985d24d9620d58",
-    "darwin-64": "aaf574fa910721fdc653519a2ca8ecf3d4e9b06213167ad630fcb9e18d329af4",
+    "windows-64": "63eef38590c7d777d75d2f8384fdfb03d47b1226c5f0bff066d97dcaacff8998",
+    "linux-64": "77cc0200c13a4c7d42363cddc76befdf8b8a19225bf48f8bcce8914d12cb1eae",
+    "darwin-64": "f81a99689507de4c177aeb0a93c1442962619747184b91b9498cbd78242695d1",
 }
 
 EXIV2_RELPATH = {
-    "windows-64": f"exiv2-{EXIV2_VERSION}-2019msvc64/bin/exiv2.exe",
+    "windows-64": f"exiv2-{EXIV2_VERSION}-2017msvc64/bin/exiv2.exe",
     "linux-64": f"exiv2-{EXIV2_VERSION}-Linux64/bin/exiv2",
     "darwin-64": f"exiv2-{EXIV2_VERSION}-Darwin/bin/exiv2",
 }
 
 EXIV2JSON_RELPATH = {
-    "windows-64": f"exiv2-{EXIV2_VERSION}-2019msvc64/bin/exiv2json.exe",
+    "windows-64": f"exiv2-{EXIV2_VERSION}-2017msvc64/bin/exiv2json.exe",
     "linux-64": f"exiv2-{EXIV2_VERSION}-Linux64/bin/exiv2json",
     "darwin-64": f"exiv2-{EXIV2_VERSION}-Darwin/bin/exiv2json",
 }
@@ -118,7 +102,6 @@ def install():
     with ThreadPoolExecutor(max_workers=6) as p:
         p.submit(exiv2_install)
         p.submit(fpcalc_install)
-        p.submit(ffprobe_install)
         p.submit(ffmpeg_install)
         p.submit(tika_install)
         p.submit(ipfs_install)
@@ -146,6 +129,7 @@ def extract(archive):  # pragma: no cover
     elif archive.endswith("tar.gz"):
         with tarfile.open(archive, "r:gz") as tar_file:
             tar_file.extractall(Path(archive).parent.absolute())
+    os.unlink(archive)
 
 
 ########################################################################################
@@ -196,6 +180,17 @@ def ipfs_version_info():  # pragma: no cover
         return "IPFS not installed"
 
 
+def run_ipfs(args: List[str]):
+    cmd = [ipfs_bin()] + args
+    try:
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    except FileNotFoundError:  # pragma: no cover
+        print("IPFS not found - installing ...")
+        ipfs_install()
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    return result
+
+
 ########################################################################################
 # Exiv2                                                                                #
 ########################################################################################
@@ -226,17 +221,6 @@ def exiv2_download():  # pragma: no cover
     return download_file(exiv2_download_url(), checksum=b3)
 
 
-def exiv2_extract(archive):  # pragma: no cover
-
-    if archive.endswith(".zip"):
-        with zipfile.ZipFile(archive, "r") as zip_file:
-            zip_file.extractall(Path(archive).parent.absolute())
-
-    elif archive.endswith("tar.gz"):
-        with tarfile.open(archive, "r:gz") as tar_file:
-            tar_file.extractall(Path(archive).parent.absolute())
-
-
 def exiv2_install():  # pragma: no cover
     """Install exiv2 command line tool and return path to executable."""
     if exiv2_is_installed():
@@ -244,7 +228,7 @@ def exiv2_install():  # pragma: no cover
         return exiv2_bin()
     log.critical("installing exiv2")
     archive_path = exiv2_download()
-    exiv2_extract(archive_path)
+    extract(archive_path)
     st = os.stat(exiv2_bin())
     os.chmod(exiv2_bin(), st.st_mode | stat.S_IEXEC)
     st = os.stat(exiv2json_bin())
@@ -268,11 +252,33 @@ def exiv2_version_info():  # pragma: no cover
             [exiv2_bin(), "--version"], stdout=subprocess.PIPE, stderr=subprocess.STDOUT
         )
         vi = r.stdout.decode(sys.stdout.encoding)
-        return vi
+        return vi.splitlines()[0]
     except FileNotFoundError:
         return "exiv2 not installed"
 
 
+def run_exiv2(args: List[str]):
+    cmd = [exiv2_bin()] + args
+    try:
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    except FileNotFoundError:  # pragma: no cover
+        print("EXIV2 not found - installing ...")
+        exiv2_install()
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    return result
+
+
+def run_exiv2json(args: List[str]):
+    cmd = [exiv2json_bin()] + args
+    try:
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    except FileNotFoundError:  # pragma: no cover
+        print("EXIV2 not found - installing ...")
+        exiv2_install()
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    return result
+
+
 ########################################################################################
 # Fpcalc                                                                               #
 ########################################################################################
@@ -321,6 +327,7 @@ def fpcalc_extract(archive):  # pragma: no cover
                     target = open(fpcalc_bin(), "wb")
                     with source, target:
                         shutil.copyfileobj(source, target)
+    os.unlink(archive)
 
 
 def fpcalc_install():  # pragma: no cover
@@ -345,65 +352,15 @@ def fpcalc_version_info():  # pragma: no cover
         return "FPCALC not installed"
 
 
-########################################################################################
-# ffprobe                                                                              #
-########################################################################################
-
-
-def ffprobe_download_url():
-    """Return system dependant download url."""
-    return FFPROBE_URLS[system_tag()]
-
-
-def ffprobe_bin() -> str:
-    """Returns local path to ffprobe executable."""
-    path = os.path.join(idk.dirs.user_data_dir, "ffprobe-{}".format(FFPROBE_VERSION))
-    if system() == "Windows":
-        path += ".exe"
-    return path
-
-
-def ffprobe_download():  # pragma: no cover
-    """Download ffprobe and return path to archive file."""
-    b3 = FFPROBE_CHECKSUMS.get(system_tag())
-    return download_file(ffprobe_download_url(), checksum=b3)
-
-
-def ffprobe_extract(archive: str):  # pragma: no cover
-    """Extract ffprobe from archive."""
-    fname = "ffprobe.exe" if system() == "Windows" else "ffprobe"
-    with zipfile.ZipFile(archive) as zip_file:
-        with zip_file.open(fname) as zf, open(ffprobe_bin(), "wb") as lf:
-            shutil.copyfileobj(zf, lf)
-
-
-def ffprobe_install():  # pragma: no cover
-    """Install ffprobe command line tool and return path to executable."""
-    if is_installed(ffprobe_bin()):
-        log.debug("ffprobe is already installed")
-        return ffprobe_bin()
-    log.critical("installing ffprobe")
-    archive_path = ffprobe_download()
-    ffprobe_extract(archive_path)
-    st = os.stat(ffprobe_bin())
-    os.chmod(ffprobe_bin(), st.st_mode | stat.S_IEXEC)
-    return ffprobe_bin()
-
-
-def ffprobe_version_info():  # pragma: no cover
-    """Get ffprobe version."""
+def run_fpcalc(args: List[str]):
+    cmd = [fpcalc_bin()] + args
     try:
-        r = subprocess.run([ffprobe_bin(), "-version"], stdout=subprocess.PIPE)
-        return (
-            r.stdout.decode("utf-8")
-            .strip()
-            .splitlines()[0]
-            .split()[2]
-            .rstrip("-static")
-            .rstrip("-tessu")
-        )
-    except FileNotFoundError:
-        return "ffprobe not installed"
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    except FileNotFoundError:  # pragma: no cover
+        print("FPCALC not found - installing ...")
+        fpcalc_install()
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    return result
 
 
 ########################################################################################
@@ -431,11 +388,12 @@ def ffmpeg_download():  # pragma: no cover
 
 
 def ffmpeg_extract(archive: str):  # pragma: no cover
-    """Extract ffprobe from archive."""
+    """Extract ffmpeg from archive."""
     fname = "ffmpeg.exe" if system() == "Windows" else "ffmpeg"
     with zipfile.ZipFile(archive) as zip_file:
         with zip_file.open(fname) as zf, open(ffmpeg_bin(), "wb") as lf:
             shutil.copyfileobj(zf, lf)
+    os.unlink(archive)
 
 
 def ffmpeg_install():  # pragma: no cover
@@ -467,6 +425,18 @@ def ffmpeg_version_info():  # pragma: no cover
         return "ffmpeg not installed"
 
 
+def run_ffmpeg(args: List[str]):
+    """Run ffmpeg command with `args`"""
+    cmd = [ffmpeg_bin()] + args
+    try:
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    except FileNotFoundError:  # pragma: no cover
+        print("FFMPEG not found - installing ...")
+        ffmpeg_install()
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    return result
+
+
 ########################################################################################
 # Java                                                                                 #
 ########################################################################################
@@ -568,6 +538,18 @@ def tika_version_info():  # pragma: no cover
         return "Tika not installed"
 
 
+def run_tika(args: List[str]):
+    cmd = [java_bin(), "-jar", tika_bin()] + args
+    try:
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    except Exception as e:  # pragma: no cover
+        print(f"TIKA error - {e}")
+        print(f"Re-Installing TIKA ...")
+        tika_install()
+        result = subprocess.run(cmd, capture_output=True, check=True)
+    return result
+
+
 def download_file(url, checksum):  # pragma: no cover
     # type: (str, str) -> str
     """Download file to app directory and return path to downloaded file."""
